/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("panel.errors"); 

BookmarkTools.panel.ID_ERRORTREE = 'bookmarkTools.pane.errorTree';
BookmarkTools.panel.ID_OTHERTREE = 'bookmarkTools.pane.otherTree';
BookmarkTools.panel.ID_ERROR_COUNT = 'bookmarkTools.pane.errorCtr';
BookmarkTools.panel.ID_OTHER_COUNT = 'bookmarkTools.pane.otherCtr';

/**
	Namespace of 'loose' functions to handle broken bookmarks.
	@namespace 
   */
BookmarkTools.panel.errors = {

	updateView : function(status, statusText, bookmark, option, key){
		var tree_error = BookmarkTools.$(BookmarkTools.panel.ID_ERRORTREE);
		var tree_other = BookmarkTools.$(BookmarkTools.panel.ID_OTHERTREE);

		if(BookmarkTools.utils.NumberUtils.isPositive(status)){
			switch(option){
				case 0 : tree_error.add(bookmark, status, statusText, key); break;
				case 1 : 
					if(status >= 500)
						tree_error.add(bookmark, status, statusText, key);
					break;
				case 2 : 
					if(status >= 400 && status < 500)
						tree_error.add(bookmark, status, statusText, key);
					break;
				default : throw 'Error: Unknown view option: ' + option;
			}
		}
		switch(status){
			case -400 : tree_other.add(bookmark, status, statusText); break;
			case -401 : tree_other.add(bookmark, status, statusText); break;
			case -402 : tree_other.add(bookmark, status, statusText); break;
			case -403 : tree_other.add(bookmark, status, statusText); break;
			case -404 : tree_other.add(bookmark, status, statusText); break;
			case -405 : tree_other.add(bookmark, status, statusText); break;
			case -406 : tree_other.add(bookmark, status, statusText); break;
			default: break;
		}
		
		BookmarkTools.$(BookmarkTools.panel.ID_ERROR_COUNT).value = "(" + tree_error.size + ")";
		BookmarkTools.$(BookmarkTools.panel.ID_OTHER_COUNT).value = "(" + tree_other.size + ")";
	},
	
	loadErrorPrefs : function(){ 
		BookmarkTools.$(BookmarkTools.panel.ID_ERRORTREE).clear(); 
		var tree = BookmarkTools.$(BookmarkTools.panel.ID_OTHERTREE); 
		tree.hideColumn(1); 
		tree.clear(); 

		var errors = BookmarkTools.utils.Prefs.getBranch('invBm').toHash(); 
		var bookmarks = BookmarkTools.common.Book.getResourceHash(); 

		var index = BookmarkTools.utils.Prefs.getInt('errorView');
		var radioGroup = BookmarkTools.$('bookmarkTools.pane.httpErrors');
		for(var i = 0, l = radioGroup.itemCount ;i < l; i++)
			radioGroup.getItemAtIndex(i).setAttribute('selected', i == index);
		
		errors.each(function(key, err){
			if(bookmarks.contains(err.uri)){
				BookmarkTools.panel.errors.updateView(
					err.status, 
					err.statusText, 
					bookmarks.get(err.uri), 
					index, key
				);
			}else {
				BookmarkTools.utils.Prefs.remove(key);
			}
		});
	},
	
	doDiscard : function(){
		var tree = null;
		var itemCount = null;

		var errorType = BookmarkTools.utils.Prefs.getInt('errorType');
		switch(errorType){
			case 0 : 
				tree = BookmarkTools.$(BookmarkTools.panel.ID_ERRORTREE);
				itemCount = BookmarkTools.$(BookmarkTools.panel.ID_ERROR_COUNT);
				break;
			case 1 :
				tree = BookmarkTools.$(BookmarkTools.panel.ID_OTHERTREE);
				itemCount = BookmarkTools.$(BookmarkTools.panel.ID_OTHER_COUNT);
				break;
			default: throw 'Unknown error type: ' + errorType;
		}
		
		var itemSet = tree.selection;
		tree.unselect();
		
		var bookmarks = BookmarkTools.common.Book.getResourceHash();
		var errors = BookmarkTools.utils.Prefs.getBranch('invBm').toHash();
		
		while(BookmarkTools.utils.ArrayUtils.isNotEmpty(itemSet)){
			var item = itemSet.pop();
			var url = tree.getItemURL(item);
			var key = tree.getItemKey(item);
			
			//remove from tree/hashmap
			tree.remove(item); 
			
			if(bookmarks.contains(url))
				BookmarkTools.common.Book.removeBookmark(bookmarks.remove(url));
			
			if(errors.contains(key)){
				BookmarkTools.utils.Prefs.remove(key);
				errors.remove(key);
			}
		}
		
		itemCount.value = '(' + tree.size + ')';
	}

};
