/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

BookmarkTools.namespace("indexing");

BookmarkTools.indexing.Document = {

	jsLUT : null,
	htmlLUT : null,
	cssLUT : null,
	cssAttrLUT : null,
	htmlAttrLUT : null,
	generalLUT : null,

	init : function(){
		var entries = BookmarkTools.utils.FileIO.listFiles(
			"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/stopwords/other"
		);
		
		var filenames = {
			"javascript" : 0, 
			"html" : 1, 
			"css" : 2, 
			"htmlAttributes" : 3, 
			"cssAttributes" : 4,
			"general" : 5
		};
		
		while(entries.hasMoreElements()){
			var file = entries.getNext().QueryInterface(Ci.nsIFile);
			var filename = file.leafName.replace(/\.txt/gi, "");
			if(BookmarkTools.utils.ObjectUtils.isNotUndefined(filenames[filename])){
				var map = BookmarkTools.utils.FileIO.readFileLineByLine(file);
				switch(filename){
					case "javascript" : this.jsLUT = map; break;
					case "html" : this.htmlLUT = map; break;
					case "css" : this.cssLUT = map; break;
					case "htmlAttributes" : this.htmlAttrLUT = map; break;
					case "cssAttributes" : this.cssAttrLUT = map; break;
					case "general" : this.generalLUT = map; break;
				}
			}
		}
	},
	
	_init : function(){
		this.jsLUT = null;
		this.htmlLUT = null;
		this.cssLUT = null;
		this.htmlAttrLUT = null;
		this.cssAttrLUT = null;
		this.generalLUT = null;
	},
	
	_stripLangAttribute : function(tags){
		//remove <a></a> tags which can refer to sub pages
		//with different language
		tags = tags.replace(
			BookmarkTools.indexing.PATTERN_TAG_A,
			BookmarkTools.utils.StringUtils.EMPTY
		);
		
		//remove <meta /> tags which can specify authors or keys
		//with different languages
		tags = tags.replace(
			BookmarkTools.indexing.PATTERN_TAG_META,
			BookmarkTools.utils.StringUtils.EMPTY
		);
	
		var matches = tags.match(BookmarkTools.indexing.PATTERN_ATTR_LANG);
		var i, match, tmp, languages = [];

		if(matches != null){
			for(i = 0, l = matches.length ;i < l; i++){
				match = matches[i];
				
				match = match.replace(
					BookmarkTools.indexing.PATTERN_WHITESPACES, 
					BookmarkTools.utils.StringUtils.EMPTY
				);
				
				match = match.replace(
					BookmarkTools.indexing.PATTERN_QUOTES, 
					BookmarkTools.utils.StringUtils.EMPTY
				);
	
				tmp = match.substring(5).split(',');
				
				languages = languages.concat(tmp);
			}
		}
		
		if(BookmarkTools.utils.ArrayUtils.isNotEmpty(languages)){
			//lowercase all entries and convert language names to iso639-1
			//shortcuts (e.g. english to en)
			languages = languages.map(BookmarkTools.indexing.Language.convert);
		}
		
		return languages;
	},
	
	_stripLangMeta : function(tags){
		var metaTags = tags.match(BookmarkTools.indexing.PATTERN_TAG_META);
		var i, http_equiv, meta, content, tmp, langName, langName2;
		
		var languages = [];
		
		if(metaTags != null){
			for(i = 0, l = metaTags.length ;i < l; i++){
				meta = metaTags[i];
				
				//try to strip http-equiv="content-language" attributes
				http_equiv = meta.match(BookmarkTools.indexing.PATTERN_ATTR_HTTP_EQUIV);
				
				//tries to find the most common name attribute that specifies
				//the content language (name="language")
				langName = meta.match(BookmarkTools.indexing.PATTERN_ATTR_NAME_LANG);
				
				//tries to find name="content-language" attributes
				langName2 = meta.match(BookmarkTools.indexing.PATTERN_ATTR_NAME_LANG_2);
				
				if(http_equiv != null || langName != null || langName2 != null){
					content = meta.match(BookmarkTools.indexing.PATTERN_ATTR_CONTENT);
					if(content != null){
						tmp = content[0].replace(
							BookmarkTools.indexing.PATTERN_WHITESPACES, 
							BookmarkTools.utils.StringUtils.EMPTY
						);
						
						tmp = tmp.replace(
							BookmarkTools.indexing.PATTERN_QUOTES, 
							BookmarkTools.utils.StringUtils.EMPTY
						);
						
						tmp = tmp.substring(8);
						languages = languages.concat(tmp.split(','));
					}
				}
			}
		}
		
		if(BookmarkTools.utils.ArrayUtils.isNotEmpty(languages)){
			//lowercase all entries and convert language names to iso639-1
			//shortcuts (e.g. english to en)
			languages = languages.map(BookmarkTools.indexing.Language.convert);
		}
		
		return languages;
	},
	
	stripLanguages : function(text){
		text = text.replace(
			BookmarkTools.indexing.PATTERN_WHITESPACES, 
			BookmarkTools.utils.StringUtils.SPACE
		);
		
		var tags = text.match(BookmarkTools.indexing.PATTERN_TAG_ANY).join(
			BookmarkTools.utils.StringUtils.EMPTY
		);
		
		var languageMeta = this._stripLangMeta(tags);
		var languageAttr = this._stripLangAttribute(tags);

		var languages = BookmarkTools.utils.ArrayUtils.merge(
			languageMeta, languageAttr
		);

		return languages;
	},
	
	_stripAnkerTag : function(text){
		var tags = text.match(BookmarkTools.indexing.PATTERN_TAG_A);
		var i, l = tags.length, tag, href, label, url_stub;
		
		var a = [];
		
		for(i = 0 ;i < l; i++){
			tag = tags[i];
			href = tag.match(BookmarkTools.indexing.PATTERN_ATTR_HREF);
			
			//href should not be empty, to avoid errors this
			//case is checked any time
			if(BookmarkTools.utils.ArrayUtils.isNotEmpty(href)){
				
				//strip <a ..>label</a>
				label = tag.replace(
					BookmarkTools.indexing.PATTERN_TAG_ANY, 
					BookmarkTools.utils.StringUtils.EMPTY
				);
				
				//strip <a href="url-stub">..</a>
				url_stub = href[0];
				
				//remove whitespaces
				url_stub = url_stub.replace(
					BookmarkTools.indexing.PATTERN_WHITESPACES, 
					BookmarkTools.utils.StringUtils.EMPTY
				);
				
				//remove any kind of quotes
				url_stub = url_stub.replace(
					BookmarkTools.indexing.PATTERN_QUOTES, 
					BookmarkTools.utils.StringUtils.EMPTY
				);
				
				// remove href=
				url_stup = url_stup.substring(5);
				
				a.push(
					{url_stup : url_stup, label : label}
				);
			}
		}
	},
	
	/**
		Strips script tags of well-formed html or xhtml strings.
		@param {string} text valid markup string
		@return striped string
	*/
	stripScripts : function(text){
		if(!BookmarkTools.utils.ObjectUtils.isString(text))
			throw 'ERROR: Invalid parameter: ' + text;
			
		return text.replace(new RegExp(BookmarkTools.indexing.PATTERN_TAG_SCRIPT, 'img'), BookmarkTools.utils.StringUtils.EMPTY);
	},
	
	stripInitialLine : function(text){
		if(!BookmarkTools.utils.ObjectUtils.isString(text))
			throw 'ERROR: Invalid parameter: ' + text;
	
		return text.replace(
			BookmarkTools.indexing.PATTERN_TAG_XML, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	},
	
	stripDocType : function(text){
		if(!BookmarkTools.utils.ObjectUtils.isString(text))
			throw 'ERROR: Invalid parameter: ' + text;
	
		return text.replace(
			BookmarkTools.indexing.PATTERN_TAG_DOCTYPE, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	},
	
	/**
		Strips tags of well-formed xml, html or xhtml strings.
		@param {string} text valid markup string
		@return striped string
	*/
	stripTags : function(text){
		if(!BookmarkTools.utils.ObjectUtils.isString(text))
			throw 'ERROR: Invalid parameter: ' + text;
			
		return text.replace(
			BookmarkTools.indexing.PATTERN_TAG_ANY, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	},
	
	stripLUT : function(text){
		var words = new BookmarkTools.indexing.LazyTokenizer().tokenize(text);
		var filtered = [], i, word, index = 0;

		for(i = 0, l = words.length ;i < l; i++){
			word = words[i];
			if(!this.jsLUT.contains(word) && !this.htmlLUT.contains(word) && !this.cssLUT.contains(word) && 
				!this.htmlAttrLUT.contains(word) && !this.cssAttrLUT.contains(word) && !this.generalLUT.contains(word)){
				filtered[index++] = word;
			}
		}
		
		return filtered.join(BookmarkTools.utils.StringUtils.SPACE);
	}

};
