/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is bookmarktools.
 *
 * The Initial Developer of the Original Code is
 * Maik Gollnick <maik.gollnick@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2008
 * the Initial Developer. All Rights Reserved.
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
 
BookmarkTools.namespace("indexing");

BookmarkTools.include("parser/bookmarkToolsTokenizer.js");
BookmarkTools.include("parser/bookmarkToolsNGram.js");
BookmarkTools.include("parser/bookmarkToolsDocument.js");

/*
for(var language in BookmarkTools.indexing.Language.supportedLanguages){
	var file = BookmarkTools.utils.FileIO.newFile(
		"extensions/" + BOOKMARKTOOLS_EXTENSION + "/resources/training/" + language + ".txt"
	);
	if(file.exists()){
		var text = BookmarkTools.utils.FileIO.readFile(file);
		BookmarkTools.indexing.NGram.addCategory(text, language);
	}
}

BookmarkTools.indexing.NGram.writeTrainingData();
*/

BookmarkTools.indexing.Encoding = {

	/*
		Returns true if the given mimetype is supported, otherwise false.
		@pre typeof mimeType != 'undefined' && mimeType != null && mimeType.toLowerCase()
	*/
	supportedMimeType : function(mimeType){
		switch(mimeType){
			case "application/rtf" : return true;
			case "application/xhtml+xml" : return true;
			case "application/xml" : return true;
			case "application/x-javascript" : return true;
			case "application/rss+xml" : return true;
			case "text/comma-separated-values" : return true;
			case "text/css" : return true;
			case "text/html" : return true;
			case "text/javascript" : return true;
			case "text/plain" : return true;
			case "text/rtf" : return true;
			case "text/tab-separated-values" : return true;
			case "text/xml" : return true;
			default : return false;
		}
	},

	/*
		Returns true if the given charset is supported, otherwise false.
		@pre typeof charSet != 'undefined' && charSet != null && charSet.toLowerCase()
	*/
	supportedCharSet : function(charSet){
		switch(charSet){
			case "charset=us-ascii" : return true;
			case "charset=utf-8" : return true;
			case "charset=iso-8859-1" : return true;
			case "charset=iso-8859-2" : return true;
			case "charset=iso-8859-3" : return true;
			case "charset=iso-8859-4" : return true;
			default : return false;
		}
	},
	
	isRTF : function(mimeType){
		return mimeType == "application/rtf" || mimeType == "text/rtf";
	},
	
	isHTML : function(mimeType){
		return mimeType == "text/html";
	},
	
	isXML : function(mimeType){
		return mimeType == "application/xhtml+xml" || 
			mimeType == "application/xml" ||
			mimeType == "application/rss+xml" ||
			mimeType == "text/xml";
	},
	
	isPLAIN : function(mimeType){
		return mimeType == "application/x-javascript" || 
			mimeType == "text/comma-separated-values" ||
			mimeType == "text/css" ||
			mimeType == "text/javascript" ||
			mimeType == "text/plain" ||
			mimeType == "text/tab-separated-values";
	},
	
	/*
		Charset independend entity filter.
	*/
	maskedEntity : function(entity){
		switch(entity){
			case "&Agrave;" : // 
			case "&Aacute;" : // 
			case "&Acirc;" : // 
			case "&Atilde;" : // 
			case "&Auml;" : // 
			case "&Aring;" : // 
				return 'A';
			case "&AElig;" : // 
				return "AE";
			case "&Ccedil;" : // 
				return 'C';
			case "&Egrave;" : // 
			case "&Eacute;" : // 
			case "&Ecirc;" : // 
			case "&Euml;" : // 
				return 'E';
			case "&Igrave;" : // 
			case "&Iacute;" : // 
			case "&Icirc;" : // 
			case "&Iuml;" : // 
				return 'I';
			case "&ETH;" : // 
				return 'D';
			case "&Ntilde;" : // 
				return 'N';
			case "&Ograve;" : // 
			case "&Oacute;" : // 
			case "&Ocirc;" : // 
			case "&Otilde;" : // 
			case "&Ouml;" : // 
			case "&Oslash;" : // 
				return 'O';
			case "&THORN;" : // 
				return "TH";
			case "&Ugrave;" : // 
			case "&Uacute;" : // 
			case "&Ucirc;" : // 
			case "&Uuml;" : // 
				return 'U';
			case "&Yacute;" : // 
				return 'Y';
			case "&agrave;" : // 
			case "&aacute;" : // 
			case "&acirc;" : // 
			case "&atilde;" : // 
			case "&auml;" : // 
			case "&aring;" : // 
				return 'a';
			case "&aelig;" : // 
				return "ae";
			case "&ccedil;" : // 
				return 'c';
			case "&egrave;" : // 
			case "&eacute;" : // 
			case "&ecirc;" : // 
			case "&euml;" : // 
				return 'e';
			case "&igrave;" : // 
			case "&iacute;" : // 
			case "&icirc;" : // 
			case "&iuml;" : // 
				return 'i';
			case "&eth;" : // 
				return 'd';
			case "&ntilde;" : // 
				return 'n';
			case "&ograve;" : // 
			case "&oacute;" : // 
			case "&ocirc;" : // 
			case "&otilde;" : // 
			case "&ouml;" : // 
			case "&oslash;" : // 
				return 'o';
			case "&szlig;" : // 
				return "ss";
			case "&thorn;" : // 
				return "th";
			case "&ugrave;" : // 
			case "&uacute;" : // 
			case "&ucirc;" : // 
			case "&uuml;" : // 
				return 'u';
			case "&yacute;" : // 
			case "&yuml;" : // 
				return 'y';
			default : break;
		}
		return null;
	},
	
	/*
		Filter for standard entities.
	*/
	standardEntity : function(entity){
		switch(entity){
			case "&quot;" : return '"';
			case "&#34;" : return '"';
			case "&amp;" : return '&';
			case "&#38;" : return '&';
			case "&lt;" : return '<';
			case "&#60;" : return '<';
			case "&gt;" : return '>';
			case "&#62;" : return '>';
			default : break;
		}
		return null;
	},
	
	/*
		Extracts all document entities as array.
	*/
	extractEntities : function(markup){
		var match = markup.match(/&([a-zA-Z0-9#]+);/gi);
		return match ? match : null;
	},
	
	/*
		Filters all known entities with ascii characters, otherwise the unknown
		entity is replaced by an empty string.
	*/
	filterEntities : function(responseText){
		var entities = BookmarkTools.indexing.Encoding.extractEntities(responseText);
		var i, entity, standardEntity, maskedEntity, replaceEntity;
		
		if(entities != null){
			for(i = 0, l = entities.length ;i < l; i++){
				entity = entities[i];
				standardEntity = BookmarkTools.indexing.Encoding.standardEntity(entity);
				maskedEntity = BookmarkTools.indexing.Encoding.maskedEntity(entity);
				
				if(standardEntity != null || maskedEntity != null){
					replaceEntity = maskedEntity != null ? maskedEntity : standardEntity;
					responseText = responseText.replace(entity, replaceEntity);
				}else {
					responseText = responseText.replace(entity, BookmarkTools.utils.StringUtils.EMPTY);
				}
			}
		}
		return responseText;
	}
	
};

BookmarkTools.indexing.Parser = {

	tidy : null,

	init : function(){
		var instTidy = Cc["@bookmarktools.mozdev.org/repair/tidy;1"].createInstance();
		this.tidy = instTidy.QueryInterface(Ci.ITidy);

		BookmarkTools.indexing.Document.init();
		BookmarkTools.indexing.NGram.readTrainingData();
		BookmarkTools.indexing.Language.init();
		BookmarkTools.indexing.Stopwords.init();
	},
	
	_init : function(){
		this.tidy = null;
	
		BookmarkTools.indexing.Document._init();
		BookmarkTools.indexing.NGram.categories = [];
		BookmarkTools.indexing.Language._init();
		BookmarkTools.indexing.Stopwords._init();
	},

	removeWhiteSpaces : function(content){
		return content.replace(
			BookmarkTools.indexing.PATTERN_WHITESPACES, 
			BookmarkTools.utils.StringUtils.EMPTY
		);
	},
	
	parseRTF : function(responseText){
		responseText = responseText.replace(/\{?\\[a-zA-Z0-9*-]+\s*/gi, BookmarkTools.utils.StringUtils.EMPTY);
		responseText = responseText.replace(/\s*[a-zA-Z\s]+;\}/gi, BookmarkTools.utils.StringUtils.EMPTY);
		return responseText.replace(/\}/gi, BookmarkTools.utils.StringUtils.EMPTY);
	},

	parse : function(mimeType, charSet, responseText){
		var words = null;
		var languages = [];
		
		var tokenizer = new BookmarkTools.indexing.WhitespaceTokenizer();
		
		if(BookmarkTools.indexing.Encoding.isPLAIN(mimeType)){
			words = tokenizer.tokenize(responseText);
		}else {
			if(BookmarkTools.indexing.Encoding.isRTF(mimeType)){
				var rtfText = BookmarkTools.indexing.Parser.parseRTF(responseText);
				words = tokenizer.tokenize(rtfText);
			}else {
				if(BookmarkTools.indexing.Encoding.isHTML(mimeType) || BookmarkTools.indexing.Encoding.isXML(mimeType)){
					var response = BookmarkTools.indexing.Parser.tidy.cleanup(responseText);
					
					var error = response == "Tidy: no output";
					var text = error ? responseText : response;
					
					/* common strip */
					text = BookmarkTools.indexing.Document.stripInitialLine(text);
					text = BookmarkTools.indexing.Document.stripDocType(text);
					text = BookmarkTools.indexing.Document.stripScripts(text);
					text = BookmarkTools.indexing.Encoding.filterEntities(text);
					
					if(error){
						text = BookmarkTools.indexing.Document.stripLUT(text);
					}else {
						text = BookmarkTools.indexing.Document.stripTags(text); 
					}
					
					words = tokenizer.tokenize(text);

					//try to strip language from document (e.g. looking for lang attribute
					//or meta tags with http-equiv="content-language")
					languages = BookmarkTools.indexing.Document.stripLanguages(
						responseText
					);
		
					if(!error){
						//if no languages are available try to classify document via ngram
						if(BookmarkTools.utils.ArrayUtils.isEmpty(languages)){
							languages[0] = BookmarkTools.indexing.NGram.classifyDocument(words);
						}
					}
					
					//cascade stopword filter
					var i, l = languages.length, language;
					for(i = 0 ;i < l; i++){
						language = languages[i];
						words = BookmarkTools.indexing.Stopwords.filter(words, language);
					}
				}else {
					throw "Error(Parser): Unsupported mimetype: " + mimeType;
				}
			}
		}
		
		return {
			words : words,
			languages : languages
		}
	}
	
}; 
